"""One of the example files showing relative ordering between modules.
See https://pytest-order.readthedocs.io/en/stable/#referencing-of-tests-in-other-classes-or-modules  # noqa: E501
"""


def test_1():
    pass


def test_2():
    pass
